@@ -358,7 +358,7 @@ getCharsInModel <- function(model) {
     } else if (model == "mk_rs1nf") {
         chars <- colnames(MK_RS1NF_Distance)
     } else if (model == "mk_rs5nf") {
-        chars <-rownames(MK_RS1NF@targeting)
+        chars <- rownames(MK_RS5NF@targeting)
     } else if (model == "hs1f_compat") {
         chars <- colnames(HS1F_Compat)
     } else if (model == "m1n_compat") {
@@ -433,8 +433,8 @@ nearestDist <- function(sequences, model=c("ham", "aa", "hh_s1f", "hh_s5f", "mk_
     
     # corresponding crossGroups values for seq_uniq
     if (!is.null(crossGroups)) {
-        stopifnot( all.equal(sequences[match(seq_uniq, sequences)], 
-                             seq_uniq, check.attributes=FALSE) )
+        stopifnot( isTRUE(all.equal(sequences[match(seq_uniq, sequences)], 
+                             seq_uniq, check.attributes=FALSE)) )
         crossGroups_uniq <- crossGroups[match(seq_uniq, sequences)]
     }
     
@@ -518,7 +518,7 @@ nearestDist <- function(sequences, model=c("ham", "aa", "hh_s1f", "hh_s5f", "mk_
             dist_mat <- dist_mat / seq_length
         } else if (normalize == "mut") {
             #dist <- dist/sum(strsplit(seq1,"")[[1]] != strsplit(seq2,"")[[1]])
-            stop('Sorry! nomalize="mut" is not available.')
+            stop('Sorry! normalize="mut" is not available.')
         }
         
         ## DEBUG
@@ -587,8 +587,8 @@ nearestDist <- function(sequences, model=c("ham", "aa", "hh_s1f", "hh_s5f", "mk_
             other_idx <- match(other_seq, seq_uniq)
             this_idx <- match(sequences[i], seq_uniq)
             
-            stopifnot( all.equal( other_seq, seq_uniq[other_idx] , check.attributes=FALSE ) )
-            stopifnot( all.equal( sequences[i], seq_uniq[this_idx] , check.attributes=FALSE ) )
+            stopifnot( isTRUE(all.equal( other_seq, seq_uniq[other_idx] , check.attributes=FALSE ) ))
+            stopifnot( isTRUE(all.equal( sequences[i], seq_uniq[this_idx] , check.attributes=FALSE ) ))
             
             # the next two checks may not always be true
             # this happens when all the out-group sequences are identical to the in-group sequences
@@ -597,7 +597,7 @@ nearestDist <- function(sequences, model=c("ham", "aa", "hh_s1f", "hh_s5f", "mk_
             
             if (subSampling) {
                 # When there is subsampling, nonsquareDist returns a non-n-by-n matrix 
-                # This matrix has fewers than n rows, and exactly n cols
+                # This matrix has fewer than n rows, and exactly n cols
                 # For each unique sequence, look for its cross-group distances in its column, 
                 #     NOT in its row (because there will be fewer than n rows)
                 
@@ -643,11 +643,14 @@ nearestDist <- function(sequences, model=c("ham", "aa", "hh_s1f", "hh_s5f", "mk_
 
 #' Distance to nearest neighbor
 #'
-#' Get non-zero distance of every heavy chain (\code{IGH}) sequence (as defined by 
-#' \code{sequenceColumn}) to its nearest sequence in a partition of heavy chains sharing the same 
-#' V gene, J gene, and junction length (V-J-length), or in a partition of single cells with heavy/long chains
-#' sharing the same heavy/long chain V-J-length combination, or of single cells with heavy/long and light/short chains 
-#' sharing the same heavy/long chain V-J-length and light/short chain V-J-length combinations.
+#' Calculate the non-zero distance from each sequence to its nearest neighbor
+#' within partitions based on shared V gene, J gene, and junction length. 
+#' 
+#' The distance to nearest neighbor can be used to estimate a threshold for assigning 
+#' Ig sequences to clonal groups. A histogram of the resulting vector is often bimodal, with the 
+#' ideal threshold being a value that separates the two modes.
+#' 
+#' Refer to the details section for a more thorough description of the implementation.
 #'
 #' @param    db              data.frame containing sequence data.
 #' @param    sequenceColumn  name of the column containing the junction for grouping and for calculating
@@ -694,13 +697,13 @@ nearestDist <- function(sequences, model=c("ham", "aa", "hh_s1f", "hh_s5f", "mk_
 #'                           \code{onlyHeavy} arguments. If set to \code{NULL} then the 
 #'                           bulk sequencing data is assumed.
 #' @param    locusColumn     name of the column containing locus information. 
-#'                           Only applicable to single-cell data.
-#'                           Ignored if \code{cellIdColumn=NULL}. Valid loci values
+#'                           Valid loci values
 #'                           are "IGH", "IGI", "IGK", "IGL", "TRA", "TRB", 
 #'                           "TRD", and "TRG".
-#' @param    onlyHeavy       use only the IGH (BCR) or TRB/TRD (TCR) sequences 
+#' @param    locusValues     Loci values to focus the analysis on.
+#' @param    onlyHeavy       This is deprecated. Only IGH (BCR) or TRB/TRD (TCR) sequences will be used
 #'                           for grouping. Only applicable to single-cell data.
-#'                           Ignored if \code{cellIdColumn=NULL}. 
+#'                           Ignored if \code{cellIdColumn=NULL}.
 #'                           See \link[alakazam]{groupGenes} for further details.               
 #' @param    keepVJLgroup    logical value specifying whether to keep in the output the the column 
 #'                           column indicating grouping based on V-J-length combinations. Only applicable for
@@ -719,13 +722,22 @@ nearestDist <- function(sequences, model=c("ham", "aa", "hh_s1f", "hh_s5f", "mk_
 #'           type \code{character} if they were type \code{factor} in the input \code{db}.
 #'
 #' @details
+#' 
+#' There are two modes of operation for \code{distToNearest}: single-cell (all sequences are 
+#' single-cell data), non-single-cell (all sequences are bulk sequencing data). Mixed data, 
+#' where both single-cell and non-single-cell sequences are present in the data, is considered 
+#' a case under the single-single cell mode .
+#' 
 #' To invoke single-cell mode the \code{cellIdColumn} argument must be specified and \code{locusColumn} 
 #' must be correct. Otherwise, \code{distToNearest} will be run with bulk sequencing assumptions, 
-#' using all input sequences regardless of the values in the \code{locusColumn} column.
+#' using all input sequences regardless of the values in the \code{locusColumn} column. 
+#' 
+#' Under single-cell mode, only heavy/long chain (IGH, TRB, TRD) sequences will be 
+#' used for calculating nearest neighbor distances regardless of \code{locusValue} values in 
+#' the \code{locusColumn} field (if present). Under non-single-cell mode, 
+#' all input sequences with \code{locusValue} value(s) in the  \code{locusColumn} field will be 
+#' used for calculating nearest neighbor distances.
 #' 
-#' Under single-cell mode, only heavy/long chain (IGH, TRB, TRD) sequences will be used for calculating 
-#' nearest neighbor distances. Under non-single-cell mode, all input sequences will be used for 
-#' calculating nearest neighbor distances, regardless of the values in the \code{locusColumn} field (if present).
 #' 
 #' Values in the \code{locusColumn} must be one of \code{c("IGH", "IGI", "IGK", "IGL")} for BCR 
 #' or \code{c("TRA", "TRB", "TRD", "TRG")} for TCR sequences. Otherwise, the function returns an 
@@ -733,17 +745,13 @@ nearestDist <- function(sequences, model=c("ham", "aa", "hh_s1f", "hh_s5f", "mk_
 #' 
 #' For single-cell mode, the input format is the same as that for \link[alakazam]{groupGenes}. 
 #' Namely, each row represents a sequence/chain. Sequences/chains from the same cell are linked
-#' by a cell ID in the \code{cellIdColumn} field. In this mode, there is a choice of whether 
-#' grouping should be done by (a) using IGH (BCR) or TRB/TRD (TCR) sequences only or
-#' (b) using IGH plus IGK/IGL (BCR) or TRB/TRD plus TRA/TRG (TCR). 
-#' This is governed by the \code{onlyHeavy} argument.
+#' by a cell ID in the \code{cellIdColumn} field. Grouping will be done by using IGH (BCR) or 
+#' TRB/TRD (TCR) sequences only. The argument that allowed to include light chains, 
+#' \code{onlyHeavy}, is deprecated.
 #' 
 #' Note, \code{distToNearest} required that each cell (each unique value in \code{cellIdColumn})
 #' correspond to only a single \code{IGH} (BCR) or \code{TRB/TRD} (TCR) sequence.
 #' 
-#' The distance to nearest neighbor can be used to estimate a threshold for assigning 
-#' Ig sequences to clonal groups. A histogram of the resulting vector is often bimodal, with the 
-#' ideal threshold being a value that separates the two modes.
 #' 
 #' The following distance measures are accepted by the \code{model} parameter.
 #' 
@@ -753,22 +761,22 @@ nearestDist <- function(sequences, model=c("ham", "aa", "hh_s1f", "hh_s5f", "mk_
 #'   \item \code{"aa"}:           Single amino acid Hamming distance matrix from \link[alakazam]{getAAMatrix}.
 #'   \item \code{"hh_s1f"}:       Human single nucleotide distance matrix derived from \link{HH_S1F} with 
 #'                                \link{calcTargetingDistance}.
-#'   \item \code{"hh_s5f"}:       Human 5-mer nucleotide context distance matix derived from \link{HH_S5F} with 
+#'   \item \code{"hh_s5f"}:       Human 5-mer nucleotide context distance matrix derived from \link{HH_S5F} with 
 #'                                \link{calcTargetingDistance}.
 #'   \item \code{"mk_rs1nf"}:     Mouse single nucleotide distance matrix derived from \link{MK_RS1NF} with 
 #'                                \link{calcTargetingDistance}.
-#'   \item \code{"mk_rs5nf"}:     Mouse 5-mer nucleotide context distance matrix derived from \link{MK_RS1NF} with 
+#'   \item \code{"mk_rs5nf"}:     Mouse 5-mer nucleotide context distance matrix derived from \link{MK_RS5NF} with 
 #'                                \link{calcTargetingDistance}.
 #'   \item \code{"hs1f_compat"}:  Backwards compatible human single nucleotide distance matrix used in 
 #'                                SHazaM v0.1.4 and Change-O v0.3.3.
-#'   \item \code{"m1n_compat"}:   Backwards compatibley mouse single nucleotide distance matrix used in 
+#'   \item \code{"m1n_compat"}:   Backwards compatibility mouse single nucleotide distance matrix used in 
 #'                                SHazaM v0.1.4 and Change-O v0.3.3.
 #' }
 #' 
 #' Note on \code{NA}s: if, for a given combination of V gene, J gene, and junction length,
 #' there is only 1  heavy chain sequence (as defined by \code{sequenceColumn}), \code{NA} is 
 #' returned instead of a distance (since it has no heavy/long chain neighbor). If for a given combination 
-#' there are multiple heavy/long chain sequences but only 1 unique one, (in which case every heavy/long cahin 
+#' there are multiple heavy/long chain sequences but only 1 unique one, (in which case every heavy/long chain 
 #' sequence in this group is the de facto nearest neighbor to each other, thus giving rise to distances 
 #' of 0), \code{NA}s are returned instead of zero-distances.
 #' 
@@ -798,7 +806,8 @@ nearestDist <- function(sequences, model=c("ham", "aa", "hh_s1f", "hh_s5f", "mk_
 #' @seealso  See \link{calcTargetingDistance} for generating nucleotide distance matrices 
 #'           from a \link{TargetingModel} object. See \link{HH_S5F}, \link{HH_S1F}, 
 #'           \link{MK_RS1NF}, \link[alakazam]{getDNAMatrix}, and \link[alakazam]{getAAMatrix}
-#'           for individual model details.
+#'           for individual model details. \link[alakazam]{getLocus} to get locus
+#'           values based on allele calls.
 #' 
 #' @examples
 #' # Subset example data to one sample as a demo
@@ -827,11 +836,19 @@ distToNearest <- function(db, sequenceColumn="junction", vCallColumn="v_call", j
                           normalize=c("len", "none"), symmetry=c("avg", "min"),
                           first=TRUE, VJthenLen=TRUE, nproc=1, fields=NULL, cross=NULL, 
                           mst=FALSE, subsample=NULL, progress=FALSE,
-                          cellIdColumn=NULL, locusColumn="locus", 
+                          cellIdColumn=NULL, locusColumn="locus", locusValues=c("IGH"),
                           onlyHeavy=TRUE, keepVJLgroup=TRUE) {
+    
     # Hack for visibility of foreach index variables
     i <- NULL
     
+    # Deprecation warning for onlyHeavy.
+    # Same warning as in alakazam::groupGenes.
+    if (!onlyHeavy) {
+        warning("onlyHeavy = FALSE is deprecated. Running as if onlyHeavy = TRUE")
+        onlyHeavy <- TRUE
+    }
+        
     # Initial checks
     model <- match.arg(model)
     normalize <- match.arg(normalize)
@@ -842,9 +859,15 @@ distToNearest <- function(db, sequenceColumn="junction", vCallColumn="v_call", j
     check <- checkColumns(db, c(sequenceColumn, vCallColumn, jCallColumn, fields, cross))
     if (check != TRUE) { stop(check) }
     
+    # Check locusColumn
+    # message locusColumn is required
+    check <- checkColumns(db, locusColumn)
+    if (check != TRUE) { stop(check, ". `locusColumn`, with default value 'locus', is now a required parameter.") }
+    if (is.null(locusColumn)) { stop("`locusColumn`, is now a required parameter.") }
+    
     # Check single-cell input
     if (!is.null(cellIdColumn)) {
-        check <- checkColumns(db, c(cellIdColumn, locusColumn))
+        check <- checkColumns(db, c(cellIdColumn))
         if (check != TRUE) { stop(check) }
     }
     
@@ -857,21 +880,42 @@ distToNearest <- function(db, sequenceColumn="junction", vCallColumn="v_call", j
     # Convert sequence columns to uppercase
     db <- toupperColumns(db, c(sequenceColumn)) 
     
+    # Create new column for distance to nearest neighbor
+    db$TMP_DIST_NEAREST <- rep(NA, nrow(db))
+    db$DTN_ROW_ID <- 1:nrow(db)
+    
+    # Check valid loci
+    if (any(is.na(db[[locusColumn]]))) {
+        stop("The locus column contains NA loci annotations.")
+    }
+    
+    # check locus column contains valid values
+    # We could use the airr schema: valid_loci <- airr::RearrangementSchema['locus'][['enum']]
+    valid_loci <- c("IGH", "IGI", "IGK", "IGL", "TRA", "TRB", "TRD", "TRG") 
+    
+    seen_loci <- unique(db[[locusColumn]])
+    check <- !all(seen_loci %in% valid_loci)
+    if (check) {
+        not_valid <- paste("'",setdiff(seen_loci,valid_loci),"'", sep="",collapse=",")
+        stop("The locus column contains invalid loci annotations: ",not_valid,".")
+    }
+    
+    invalid_locus_values <- locusValues[locusValues %in% valid_loci == F]
+    if (length(invalid_locus_values)>0) {
+        stop("`locusValues` contains invalid loci annotations: ",paste(invalid_locus_values,collapse=", "),".")
+    }
+    
     # Single-cell mode?
-    if (!is.null(cellIdColumn) & !is.null(locusColumn)) {
+    if (!is.null(cellIdColumn)) {
         singleCell <- TRUE
-        
-        # check locus column
-        valid_loci <- c("IGH", "IGI", "IGK", "IGL", "TRA", "TRB", "TRD", "TRG")
-        check <- !all(unique(db[[locusColumn]]) %in% valid_loci)
-        if (check) {
-            stop("The locus column contains invalid loci annotations.")
-        }
     } else {
         singleCell <- FALSE
-    } 
+    }
     
     # Disallow multiple heavy chains per cell
+    # sequences with cell_id==NA are not considered, table's default 
+    # is useNA="no". In practice that means that distances 
+    # will be calculated between sequences of different cells.
     if (singleCell) {
         # check multiple heavy chains
         x <- sum(table(db[[cellIdColumn]][db[[locusColumn]] == "IGH"]) > 1)
@@ -891,7 +935,6 @@ distToNearest <- function(db, sequenceColumn="junction", vCallColumn="v_call", j
     }
     
     # Check for invalid characters
-    # heavy
     valid_seq <- sapply(db[[sequenceColumn]], allValidChars, getCharsInModel(model)) 
     not_valid_seq <- which(!valid_seq)
     if (length(not_valid_seq) > 0) {
@@ -900,41 +943,100 @@ distToNearest <- function(db, sequenceColumn="junction", vCallColumn="v_call", j
         db <- db[valid_seq, ]
     }
     
+    # Info for users
+    if (singleCell) {
+
+        light_chains <- sum(db[[locusColumn]] %in% c("IGH", "TRB", "TRD") == FALSE)
+        msg <- paste0("Running in single-cell mode.")
+        
+        if (light_chains > 0) {
+            msg <- paste(msg,
+                        paste0("Note: ", light_chains, 
+                        " light/short chain sequences will receive NA distances."),
+                        sep="\n")
+        }    
+        
+        not_used_loci <- setdiff(locusValues, c("IGH", "TRB", "TRD"))
+        if (length(not_used_loci)>0) {
+            this_msg <- paste0("Note: locusValues contains loci that will not be used for distance calculations: ", 
+                    paste(not_used_loci, collapse=", "), 
+                    ". Only IGH (BCR) or TRB/TRD (TCR) sequences are used for distance calculations in single-cell mode.")
+            msg <- paste(msg,this_msg, sep="\n")
+        }
+        message(msg)
+    } else {
+        message("Running in non-single-cell mode.")
+    }
     # junction length columns (prep for groupGenes)
     junc_len <- "JUNC_LEN"
     db[[junc_len]] <- stri_length(db[[sequenceColumn]])
     
+    # fields groups
+    db$DTN_TMP_FIELD <- db %>%
+        group_by(!!!rlang::syms(fields)) %>%
+        group_indices()
+    
     # create V+J grouping, or V+J+L grouping
-    if (VJthenLen) {
+    if (VJthenLen) { 
         # 2-stage partitioning using first V+J and then L
         # V+J only first
         # creates $vj_group
-        db <- groupGenes(db, v_call=vCallColumn, j_call=jCallColumn, junc_len=NULL,
+        db <- db %>%
+            ungroup() %>%
+            group_by(!!rlang::sym("DTN_TMP_FIELD")) %>%
+            do(groupGenes(.data, v_call=vCallColumn, j_call=jCallColumn, junc_len=NULL,
                          cell_id=cellIdColumn, locus=locusColumn, only_heavy=onlyHeavy,
-                         first=first)
+                         first=first)) %>%
+            ungroup()
         # L (later)  
-        group_cols <- c("vj_group", junc_len)
+        # add locusColumn to account for single cells having light chains assigned
+        # the same group as the paired heavy chain
+        group_cols <- c("vj_group", junc_len, locusColumn)
         
     } else {
         # 1-stage partitioning using V+J+L simultaneously
         # creates $vj_group
         # note that despite the name (VJ), this is based on V+J+L
-        db <- groupGenes(db, v_call=vCallColumn, j_call=jCallColumn, junc_len=junc_len,
-                         cell_id=cellIdColumn, locus=locusColumn, only_heavy=onlyHeavy,
-                         first=first)
-        group_cols <- c("vj_group")
+        db <- db %>%
+            ungroup() %>%
+            group_by(!!rlang::sym("DTN_TMP_FIELD")) %>%
+            do(groupGenes(.data, v_call=vCallColumn, j_call=jCallColumn, junc_len=junc_len,
+                          cell_id=cellIdColumn, locus=locusColumn, only_heavy=onlyHeavy,
+                          first=first)) %>%
+            ungroup()
+        
+        # add locusColumn to account for single cells having light chains assigned
+        # the same group as the paired heavy chain
+        group_cols <- c("vj_group", locusColumn)
     }
     
     # groups to use
     if (!is.null(fields)) {
         group_cols <- append(group_cols,fields)
+        # make vj_group unique across fields by pasting field group
+        db <- db %>%
+            dplyr::rowwise() %>%
+            mutate(vj_group=paste("F",!!rlang::sym("DTN_TMP_FIELD"),"_",!!rlang::sym("vj_group"), sep="", collapse = "")) %>%
+            ungroup() 
     }
+    db[['DTN_TMP_FIELD']] <- NULL
+    
+    if (any(is.na(db[["vj_group"]]))) {
+        warning("The vj_group column contains NA values corresponding to ", 
+                sum(is.na(db$vj_group)), 
+                " sequences to which alakazam::groupGenes could not assign a group.",
+                " These sequences will not be analyzed.")
+    }
+    
     # unique groups
     # not necessary but good practice to force as df and assign colnames
     # (in case group_cols has length 1; which can happen in groupBaseline)
-    uniqueGroups <- data.frame(unique(db[, group_cols]), stringsAsFactors=FALSE)
+    uniqueGroups <- data.frame(
+        unique(db[, group_cols]) %>% filter(!is.na(!!rlang::sym("vj_group"))),
+        stringsAsFactors=FALSE)
     colnames(uniqueGroups) <- group_cols
     rownames(uniqueGroups) <- NULL
+    
     # indices
     # crucial to have simplify=FALSE 
     # (otherwise won't return a list if uniqueClones has length 1)
@@ -955,10 +1057,6 @@ distToNearest <- function(db, sequenceColumn="junction", vCallColumn="v_call", j
         stopifnot( max(curIdx) <= nrow(db) )
         return(curIdx)
     }, simplify=FALSE)
-
-    # Create new column for distance to nearest neighbor
-    db$TMP_DIST_NEAREST <- rep(NA, nrow(db))
-    db$ROW_ID <- 1:nrow(db)
     
     # Create cluster of nproc size and export namespaces
     # If user wants to paralellize this function and specifies nproc > 1, then
@@ -970,13 +1068,19 @@ distToNearest <- function(db, sequenceColumn="junction", vCallColumn="v_call", j
         registerDoSEQ()
     } else if( nproc > 1 ) {
         cluster <- parallel::makeCluster(nproc, type="PSOCK")
-        registerDoParallel(cluster)
+        registerDoParallel(cluster,cores=nproc)
     } else {
         stop('Nproc must be positive.')
     }
     
     # Export groups to the clusters
     if (nproc > 1) { 
+        # This subsetting improves performance
+        required_cols <- unique(c(group_cols,cross,locusColumn,sequenceColumn,"TMP_DIST_NEAREST"))
+        db_notused_cols <- db %>%
+            select(c(!!rlang::sym("DTN_ROW_ID"), !any_of(required_cols)))
+        db <- db %>%
+            select(c(!!rlang::sym("DTN_ROW_ID"),  any_of(required_cols)))
         export_functions <- list("db",
                                  "uniqueGroupsIdx", 
                                  "cross",
@@ -998,7 +1102,8 @@ distToNearest <- function(db, sequenceColumn="junction", vCallColumn="v_call", j
                                  "pairwise5MerDist",
                                  "nonsquare5MerDist",
                                  "singleCell",
-                                 "locusColumn")
+                                 "locusColumn",
+                                 "locusValues")
         parallel::clusterExport(cluster, export_functions, envir=environment())
     }
     
@@ -1012,13 +1117,17 @@ distToNearest <- function(db, sequenceColumn="junction", vCallColumn="v_call", j
         # wrt db
         idx <- uniqueGroupsIdx[[i]]
         
-        if (singleCell) {
-            # only use IGH, TRB, TRD
-            # wrt idx
-            idxBool <- db[[locusColumn]][idx] %in% c("IGH", "TRB", "TRD")
-        } else {
-            idxBool <- rep(TRUE, length(idx))
-        }
+        # if (singleCell) {
+        #     # only use IGH, TRB, TRD
+        #     # wrt idx
+        #     idxBool <- db[[locusColumn]][idx] %in% c("IGH", "TRB", "TRD")
+        # } else {
+        #     idxBool <- rep(TRUE, length(idx))
+        # }
+        
+        # for the distance calculation use only
+        # sequences with locus values specified in `locusValues`
+        idxBool <- toupper(db[[locusColumn]][idx]) %in% locusValues
         
         db_group <- db[idx, ]
         
@@ -1051,11 +1160,22 @@ distToNearest <- function(db, sequenceColumn="junction", vCallColumn="v_call", j
     )
     
     # Convert list from foreach into a db data.frame
-    db <- do.call(rbind, list_db)
-    db <- db[order(db$ROW_ID), ]
+    if (!any(is.na(db[["vj_group"]]))) {
+        db <-do.call(rbind, list_db)
+    } else {
+        db <- bind_rows(
+            db %>% filter(is.na(!!rlang::sym("vj_group"))),
+            do.call(rbind, list_db))
+    }
+
+    # Stop the cluster and add back not used colums
+    if (nproc > 1) { 
+        parallel::stopCluster(cluster)
+        db <- db %>%
+            left_join(db_notused_cols, by= "DTN_ROW_ID")
+    }
     
-    # Stop the cluster
-    if (nproc > 1) { parallel::stopCluster(cluster) }
+    db <- db[order(db$DTN_ROW_ID), ]
     
     if (!is.null(cross)) {
         db$cross_dist_nearest <- db$TMP_DIST_NEAREST
@@ -1067,7 +1187,7 @@ distToNearest <- function(db, sequenceColumn="junction", vCallColumn="v_call", j
     if ((!VJthenLen) && keepVJLgroup) {
         db$vjl_group <- db[["vj_group"]]
     }
-    db <- db[, !(names(db) %in% c(junc_len, "vj_group", "ROW_ID", "V1", "J1","TMP_DIST_NEAREST"))]
+    db <- db[, !(names(db) %in% c(junc_len, "vj_group", "DTN_ROW_ID", "V1", "J1","TMP_DIST_NEAREST"))]
     
     return(db)
 }
@@ -1077,7 +1197,7 @@ distToNearest <- function(db, sequenceColumn="junction", vCallColumn="v_call", j
 
 #' Find distance threshold
 #'
-#' \code{findThreshold} automtically determines an optimal threshold for clonal assignment of
+#' \code{findThreshold} automatically determines an optimal threshold for clonal assignment of
 #' Ig sequences using a vector of nearest neighbor distances. It provides two alternative methods 
 #' using either a Gamma/Gaussian Mixture Model fit (\code{method="gmm"}) or kernel density 
 #' fit (\code{method="density"}).
@@ -1144,29 +1264,19 @@ distToNearest <- function(db, sequenceColumn="junction", vCallColumn="v_call", j
 #' 
 #' @examples
 #' \donttest{
-#' # Subset example data to one sample as a demo
+#' # Subset example data to 50 sequences, one sample and isotype as a demo
 #' data(ExampleDb, package="alakazam")
-#' db <- subset(ExampleDb, sample_id == "-1h")
+#' db <- subset(ExampleDb, sample_id == "-1h" & c_call=="IGHG")[1:50,]
 #' 
 #' # Use nucleotide Hamming distance and normalize by junction length
 #' db <- distToNearest(db, sequenceColumn="junction", vCallColumn="v_call",
 #'                     jCallColumn="j_call", model="ham", normalize="len", nproc=1)
 #'                             
-#' # Find threshold using the "gmm" method with optimal threshold
-#' output <- findThreshold(db$dist_nearest, method="gmm", model="gamma-gamma", cutoff="opt")
-#' plot(output, binwidth=0.02, title=paste0(output@model, "   loglk=", output@loglk))
-#' print(output)
-#'
 #' # Find threshold using the "gmm" method with user defined specificity
 #' output <- findThreshold(db$dist_nearest, method="gmm", model="gamma-gamma", 
 #'                         cutoff="user", spc=0.99)
 #' plot(output, binwidth=0.02, title=paste0(output@model, "   loglk=", output@loglk))
 #' print(output)
-#'
-#' # Find threshold using the "density" method and plot the results
-#' output <- findThreshold(db$dist_nearest, method="density")
-#' plot(output)
-#' print(output)
 #' }
 #' @export
 findThreshold <- function (distances, method=c("density", "gmm"), 
@@ -1261,12 +1371,17 @@ findThreshold <- function (distances, method=c("density", "gmm"),
  smoothValley <- function(distances) {
     # Remove NA, NaN, and infinite distances
     distances <- distances[!is.na(distances) & !is.nan(distances) & !is.infinite(distances)]
-
+    unique_distances <- unique(distances)
     # Gaussian distribution bandwidth scale parameter
     # gaussian_scaling <- (1/(4 * pi))^(1/10)
     
     # Ideal bandwidth
-    bandwidth <- h.ucv(unique(distances), 4)$h
+    if (length(unique_distances) < 3 ) {
+        stop("The `smoothValley` funcion used by the density method requires ", 
+             "at least 3 unique distance values.\n",
+             "Found distance values: ", paste(unique_distances, collapse=", "))
+    }
+    bandwidth <- h.ucv(unique_distances, 4)$h
     #bandwidth <- kedd::h.ucv(distances, 4)$h
     #bandwidth <- ks::hucv(unique(distances), deriv.order=4)
     
@@ -1576,12 +1691,12 @@ rocSpace <- function(ent, omega.gmm, mu.gmm, sigma.gmm, model, cutoff, sen, spc,
         pb <- progressBar(15)
     }
     for (i in 1:15) {
-        #itr<-1
-        key<-FALSE
-        while (!key){
-            # print(paste0(i,":",itr))
+        itr <- 1
+        max_itr <- 100
+        key <- FALSE
+        while (!key && itr <= max_itr){
             # Fit mixture Functions
-            MixModel <- try(suppressWarnings(fitdistr(na.exclude(ent), mixFunction, 
+            MixModel <- try(suppressWarnings(MASS::fitdistr(na.exclude(ent), mixFunction, 
                                      first_curve = bits[1], second_curve = bits[2], 
                                      start=list(omega = func1.0, 
                                                 func1.1 = func1.1, func1.2 = func1.2,
@@ -1594,7 +1709,7 @@ rocSpace <- function(ent, omega.gmm, mu.gmm, sigma.gmm, model, cutoff, sen, spc,
                 func1.2 <- abs(gmmfunc1.2 + sample(c(-1,1), 1)*runif(1))
                 func2.1 <- abs(gmmfunc2.1 + sample(c(-1,1), 1)*runif(1))
                 func2.2 <- abs(gmmfunc2.2 + sample(c(-1,1), 1)*runif(1))
-                #itr<-itr+1
+                itr <- itr + 1
                 next
             } else if ( (bits[1] == "norm"  & bits[2] == "gamma" & MixModel$estimate[[2]] > MixModel$estimate[[4]] * MixModel$estimate[[5]]) |
                         (bits[1] == "gamma" & bits[2] == "norm"  & MixModel$estimate[[2]] * MixModel$estimate[[3]] > MixModel$estimate[[4]]) |
@@ -1606,12 +1721,17 @@ rocSpace <- function(ent, omega.gmm, mu.gmm, sigma.gmm, model, cutoff, sen, spc,
                 func2.1 <- abs(gmmfunc2.1 + sample(c(-1,1), 1)*runif(1))
                 func2.2 <- abs(gmmfunc2.2 + sample(c(-1,1), 1)*runif(1))
                 # print("here")
-                #itr<-itr+1
+                itr <- itr + 1
                 next
             } else {
-                key<-TRUE
+                key <- TRUE
             }
         }
+        
+        # Check if we failed to find a valid fit after max iterations
+        if (!key) {
+            stop(paste0("Failed to fit after ", max_itr, " attempts in outer iteration ", i))
+        }
         # print(paste0(func, " fit done. Loglik= ", round(MixModel$loglik, digits = 2)))
         # Invoke fit parameters
         # log_lik <- round(MixModel$loglik, digits = 2)
@@ -1953,21 +2073,25 @@ plotGmmThreshold <- function(data, cross=NULL, xmin=NULL, xmax=NULL, breaks=NULL
     if (is.null(xmax)) { xmax <- NA }
     
     # Plot distToNearest distribution plus Gaussian fits
-    p <- ggplot(xdf, aes_string(x="x")) +
+    p <- ggplot(xdf, aes(x=!!rlang::sym("x"))) +
         baseTheme() + 
         xlab("Distance") + 
         ylab("Density") +
-        geom_histogram(aes_string(y="..density.."), binwidth=binwidth, 
+        geom_histogram(aes(y=after_stat(!!str2lang("density"))), binwidth=binwidth, 
                        fill="gray40", color="white") +
-        geom_line(data=fit1, aes_string(x="x", y="y"), color="darkslateblue", size=size) +
-        geom_line(data=fit2, aes_string(x="x", y="y"), color="darkslateblue", size=size) +
+        geom_line(data=fit1, aes(x=!!rlang::sym("x"), y=!!rlang::sym("y")), 
+                  color="darkslateblue", linewidth=size) +
+        geom_line(data=fit2, aes(x=!!rlang::sym("x"), y=!!rlang::sym("y")), 
+                  color="darkslateblue", linewidth=size) +
         geom_vline(xintercept=data@threshold, color="firebrick", 
-                   linetype="longdash", size=size)
+                   linetype="longdash", linewidth=size)
     
     # Add cross histogram
     if (!is.null(cross)) {
         cdf <- data.frame(x=cross[is.finite(cross)])
-        p <- p + geom_histogram(data=cdf, aes_q(x=~x, y=~-(..density..)), binwidth=binwidth, 
+        p <- p + geom_histogram(data=cdf, 
+                                aes(x=!!rlang::sym("x"), 
+                                    y=-after_stat(!!str2lang("density"))), binwidth=binwidth, 
                                 fill="gray40", color="white", position="identity") +
             scale_y_continuous(labels=abs)
     }
@@ -2062,21 +2186,30 @@ plotDensityThreshold <- function(data, cross=NULL, xmin=NULL, xmax=NULL, breaks=
     if (is.null(xmax)) { xmax <- NA }
     
     # Plot distToNearest distribution plus Gaussian fits
-    p <- ggplot(xdf, aes_string(x="x")) +
+    p <- ggplot(xdf, aes(x=!!rlang::sym("x"))) +
         baseTheme() +
         xlab("Distance") + 
         ylab("Density") +
-        geom_histogram(aes_string(y="..density.."), binwidth=binwidth, 
+        geom_histogram(aes(y=after_stat(!!str2lang("density"))), binwidth=binwidth, 
                        fill="gray40", color="white") +
-        geom_line(data=ddf, aes_string(x="x", y="y"), 
-                  color="darkslateblue", size=size) +
-        geom_vline(xintercept=data@threshold, 
-                   color="firebrick", linetype="longdash", size=size)
+        geom_line(data=ddf, 
+                  aes(x=!!rlang::sym("x"), 
+                      y=!!rlang::sym("y")), 
+                  color="darkslateblue", linewidth=size)
+
+    # Only add threshold line if threshold is not NA or infinite
+    if (!is.na(data@threshold) && is.finite(data@threshold)) {
+        p <- p + geom_vline(xintercept=data@threshold, 
+                           color="firebrick", linetype="longdash", linewidth=size)
+    }
     
     # Add cross histogram
     if (!is.null(cross)) {
         cdf <- data.frame(x=cross[is.finite(cross)])
-        p <- p + geom_histogram(data=cdf, aes_q(x=~x, y=~-(..density..)), binwidth=binwidth, 
+        p <- p + geom_histogram(data=cdf, 
+                                aes(x=!!rlang::sym("x"), 
+                                    y=-after_stat(!!str2lang("density"))), 
+                                binwidth=binwidth, 
                                 fill="gray40", color="white", position="identity") +
              scale_y_continuous(labels=abs)
     }
@@ -2105,4 +2238,3 @@ plotDensityThreshold <- function(data, cross=NULL, xmin=NULL, xmax=NULL, breaks=
         return(p)
     }
 }
-
