@@ -1,3 +1,7 @@
+#' @keywords internal
+#' @aliases shazam-package
+"_PACKAGE"
+
 # Shazam package documentation and import directives
 
 #' The shazam package
@@ -61,7 +65,7 @@
 #'                                           between BASELINe PDFs.
 #'   \item  \link{plotBaselineDensity}:      Plot the probability density functions
 #'                                           resulting from selection analysis.
-#'   \item  \link{plotBaselineSummary}:      Plot summary stastistics resulting from 
+#'   \item  \link{plotBaselineSummary}:      Plot summary statistics resulting from 
 #'                                           selection analysis.
 #' }
 #'
@@ -81,7 +85,6 @@
 #' }
 #'
 #' @name     shazam
-#' @docType  package
 #' @references
 #' \enumerate{
 #'   \item  Hershberg U, et al. Improved methods for detecting selection by mutation 
@@ -106,7 +109,7 @@
 #' @import   graphics
 #' @import   methods
 #' @import   utils
-#' @importFrom  alakazam    getAllele getGene getFamily getSegment groupGenes
+#' @importFrom  alakazam    getAllele getGene getFamily getSegment getLocus groupGenes
 #'                          getAAMatrix getDNAMatrix IUPAC_DNA
 #'                          pairwiseDist nonsquareDist pairwiseEqual 
 #'                          seqDist seqEqual
@@ -124,42 +127,38 @@
 #'                          mutate summarize
 #'                          mutate_at summarize_at
 #'                          rename transmute
-#'                          left_join
+#'                          left_join recode pull
 #' @importFrom  foreach     foreach %dopar% registerDoSEQ
 #' @importFrom  igraph      V E as_adjacency_matrix graph_from_data_frame
 #'                          vertex_attr set_vertex_attr 
 #'                          layout_as_tree V<-
 #' @importFrom  iterators   icount
-#' @importFrom  kedd        h.ucv
 #' @importFrom  KernSmooth  bkde
 #' @importFrom  lazyeval    interp
 #' @importFrom  MASS        fitdistr
 #' @importFrom  progress    progress_bar
-#' @importFrom  rlang       sym syms
+#' @importFrom  rlang       sym syms .data
 #' @importFrom  scales      log2_trans log10_trans trans_breaks trans_format
 #'                          math_format percent scientific pretty_breaks
 #' @importFrom  seqinr      c2s s2c words translate
 #' @importFrom  stats       na.omit setNames ecdf sd cor cov median mad
 #'                          approx convolve weighted.mean p.adjust
 #'                          dbeta pbeta qbeta rbeta optim optimize
-#'                          dnorm pnorm runif dgamma pgamma uniroot na.exclude
-#'                          as.dist cutree
+#'                          dnorm pnorm rbinom runif dgamma pgamma uniroot na.exclude
+#'                          as.dist cutree integrate
 #' @importFrom  stringi     stri_dup stri_flatten stri_join stri_length 
 #'                          stri_sub stri_sub_replace stri_detect_regex
 #'                          stri_count_boundaries stri_count_regex 
 #'                          stri_extract_all_regex stri_extract_first_regex  
 #'                          stri_replace_all_regex stri_replace_first_regex
-#' @importFrom  tidyr       gather spread
-#' @importFrom  tidyselect  all_of
+#' @importFrom  tidyr       gather spread pivot_wider
+#' @importFrom  tidyselect  all_of any_of
 NULL
 
 # Package loading actions
 .onAttach <- function(libname, pkgname) {
-    msg <- paste("As of v1.0.0 the AIRR Rearrangement schema is now the default file format.",
-                 "A description of the standard is available at https://docs.airr-community.org.",
-                 "The legacy Change-O format is supported through arguments to each function",
-                 "that allow the input column names to be explicitly defined.",
-                 sep="\n")
+    msg <- citation(pkgname)
+    msg <-paste(c(format(msg,"citation")),collapse="\n\n")
     packageStartupMessage(msg)
 }
 
@@ -171,7 +170,7 @@ NULL
 # Mus musculus Ig sequence data.
 #
 # @format   A symmetric matrix of nucleotide substitution distance scores with 
-#           row names and column names definition the specific subsitution.
+#           row names and column names definition the specific substitution.
 # 
 # @references
 # \enumerate{
@@ -189,7 +188,7 @@ NULL
 # human Ig sequence data.
 #
 # @format   A symmetric matrix of nucleotide substitution distance scores with 
-#           row names and column names definition the specific subsitution.
+#           row names and column names definition the specific substitution.
 # 
 # @references
 # \enumerate{
@@ -230,6 +229,10 @@ NULL
 # TODO: And what is this?
 # BAYESIAN_FITTED
 
+# IMGT-KABAT numbering mapping
+# As described hare http://www.imgt.org/IMGTScientificChart/Numbering/IMGT-Kabat_part1.html
+# CONVERT_NUM_REF
+
 # Add built-in variables to global variables environment
 utils::globalVariables(c("HH_S1F", "HKL_S1F", "MK_RS1NF",
                          "HH_S5F", "HKL_S5F", "MK_RS5NF", "U5N",
